document.addEventListener("DOMContentLoaded", () => {
    type TProfile = {
        email: string, 
        firstName: string, 
        lastName: string, 
        _id: string
    };

    type TParseGet = {
        url: string,
        isGet: boolean,
        data?: any
    };

    const posts = document.getElementById("posts");
    const firstName = document.getElementById("firstName");
    const lastName = document.getElementById("lastName");
    const inputFirstName = document.getElementById("inputFirstName");
    const inputLastName = document.getElementById("inputLastName");
    const inputEmail = document.getElementById("inputEmail");
    const inputPost = document.getElementById("inputPost");
    const formSend = document.getElementById("form_send");
    const changeProfile = document.getElementById("change_profile");
    const followUser = document.getElementById("follow_user");
    const profileModal = document.getElementById("profile_modal");
    const followUserModal = document.getElementById("follow_user_modal");
    const profileModalForm = document.getElementById("profile_modal_form");
    const followUserModalForm = document.getElementById("follow_user_modal_form");
    const values = {firstName: "", lastName: "", email: "", post: ""};


    const parseGetData = async ({url, isGet, data}: TParseGet): Promise<any> => {
        return isGet ?
        await (await fetch(url)).json() :
        await (await fetch(url, {method: "POST", headers: {"Content-Type": "application/x-www-form-urlencoded"}, body: new URLSearchParams(data)})).json();
    };

    const createElem = ({fullName, message}: {fullName: string, message: string}) => {
        const div = document.createElement("div");
        div.setAttribute("class", "post");
        div.innerHTML = `<div class="author">${fullName} said:</div><div class="message">${message}</div>`
        return div;
    };


    class Profile {
        public firstName: string = "";
        public lastName: string = "";
        public email: string = "adam@gmail.com";

        constructor() {
            this.registerProfile();
        };

        async registerProfile() {
            const res: TProfile = await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/profile`, isGet: true});
            this.firstName = res.firstName;
            this.lastName = res.lastName;
        };

        async changeProfile({firstName, lastName}: {firstName: string, lastName: string}) {
            const obj = { firstName, lastName };
            const res: TProfile = await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/profile`, isGet: false, data: obj});            
            this.firstName = res.firstName;
            this.lastName = res.lastName;
        };

        async subscribe(email: string) {
            await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/subscribe`, isGet: false, data: {email}});            
        };
    };


    class Post extends Profile {
        public posts: any;
        public lastDate: string | null = null;

        constructor() {
            super();
            this.getPosts();
            setInterval(() => {
                if(this.lastDate) {
                    this.getLastPosts();
                };
            }, 2000);
        };

        async getPosts() {
            const res = await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/posts`, isGet: true});
            this.lastDate = res.reverse().shift().datetime;

            res.forEach((item: any) => {
                posts?.append(createElem({fullName: `${item.user.firstName} ${item.user.lastName}`, message: item.message}))
            });
        };

        async getLastPosts() {
            const res = await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/posts?datetime=${this.lastDate}`, isGet: true});
            if(res.length !== 0) {
                this.lastDate = res[0].datetime
                posts?.append(createElem({fullName: `${res[0].user.firstName} ${res[0].user.lastName}`, message: res[0].message}));
            }
        };

        async createPost(message: string) {
            await parseGetData({url: `http://146.185.154.90:8000/blog/${this.email}/posts`, isGet: false, data: { message }});   
        };
    };
    

    const profile = new Profile();
    const post = new Post();

    changeProfile?.addEventListener("click", () => {
        profileModal && (profileModal.style.display = "block");
    });

    inputFirstName?.addEventListener("change", (e: any) => {
        values.firstName = e.target.value;
    });

    inputLastName?.addEventListener("change", (e: any) => {
        values.lastName = e.target.value;
    });

    inputEmail?.addEventListener("change", (e: any) => {
        values.email = e.target.value;
    });

    inputPost?.addEventListener("change", (e: any) => {
        values.post = e.target.value;
    });

    profileModalForm?.addEventListener("submit", (e) => {
        e.preventDefault();
        profile.changeProfile({firstName: values.firstName, lastName: values.lastName});
        firstName!.innerText = values.firstName;
        lastName!.innerText = values.lastName;
        profileModal!.style.display = "none";
    });

    followUser?.addEventListener("click", () => {
        followUserModal!.style.display = "block";
    });

    followUserModalForm?.addEventListener("submit", (e) => {
        e.preventDefault();
        profile.subscribe(values.email);
        followUserModal!.style.display = "none";
    });

    formSend?.addEventListener("submit", (e) => {
        e.preventDefault();
        post.createPost(values.post);
    });
});